namespace nkmidi.Meta
{
	using System;

	/// <summary>
	/// MetaEvent: ^Cxg̃f[^\钊ۃNX
	/// </summary>
	abstract public class MetaEvent
	{
		private int eventType;

		public const int EVENTTYPE_SEQUENCE_NUMBER = 0x00;
		public const int EVENTTYPE_TEXT = 0x01;
		public const int EVENTTYPE_COPYRIGHT = 0x02;
		public const int EVENTTYPE_SEQUENCE_OR_TRACK_NAME = 0x03;
		public const int EVENTTYPE_INSTRUMENT_NAME = 0x04;
		public const int EVENTTYPE_LYRICS = 0x05;
		public const int EVENTTYPE_MARKER = 0x06;
		public const int EVENTTYPE_CUE_POINT = 0x07;
		public const int EVENTTYPE_MIDI_CHANNEL_PREFIX = 0x20;
		public const int EVENTTYPE_MIDI_PORT = 0x21;
		public const int EVENTTYPE_END_OF_TRACK = 0x2F;
		public const int EVENTTYPE_SET_TEMPO = 0x51;
		public const int EVENTTYPE_SMPTE_OFFSET = 0x54;
		public const int EVENTTYPE_TIME_SIGNATURE = 0x58;
		public const int EVENTTYPE_KEY_SIGNATURE = 0x59;
		public const int EVENTTYPE_SEQUENCER_SPECIFIC = 0x7F;

		public MetaEvent(int event_type)
		{
			this.EventType = event_type;
		}

		public int EventType
		{
			get { return eventType; }
			set { eventType = value; }
		}
	}

	/// <summary>
	/// MetaSequenceNumber: Sequence Number
	/// </summary>
	public class MetaSequenceNumber : MetaEvent
	{
		private int sequenceNumber;

		public MetaSequenceNumber() : base(MetaEvent.EVENTTYPE_SEQUENCE_NUMBER)
		{
			this.SequenceNumber = 0;
		}

		public int SequenceNumber
		{
			get { return sequenceNumber; }
			set { sequenceNumber = value; }
		}
	}

	/// <summary>
	/// MetaText: Text Event
	/// </summary>
	public class MetaText : MetaEvent
	{
		private string text;

		public MetaText() : base(MetaEvent.EVENTTYPE_TEXT)
		{
			this.Text = "";
		}

		public string Text
		{
			get { return text; }
			set { text = value; }
		}
	}

	/// <summary>
	/// MetaCopyright: Copyright Notice
	/// </summary>
	public class MetaCopyright : MetaEvent
	{
		private string copyright;

		public MetaCopyright() : base(MetaEvent.EVENTTYPE_COPYRIGHT)
		{
			this.Copyright = "";
		}

		public string Copyright
		{
			get { return copyright; }
			set { copyright = value; }
		}
	}

	/// <summary>
	/// MetaSequenceOrTrackName: Sequence Name or Track Name
	/// </summary>
	public class MetaSequenceOrTrackName : MetaEvent
	{
		private string name;

		public MetaSequenceOrTrackName() : base(MetaEvent.EVENTTYPE_SEQUENCE_OR_TRACK_NAME)
		{
			this.Name = "";
		}

		public string Name
		{
			get { return name; }
			set { name = value; }
		}
	}

	/// <summary>
	/// MetaInstrumentName: Instrument Name
	/// </summary>
	public class MetaInstrumentName : MetaEvent
	{
		private string instrumentName;

		public MetaInstrumentName() : base(MetaEvent.EVENTTYPE_INSTRUMENT_NAME)
		{
			this.InstrumentName = "";
		}

		public string InstrumentName
		{
			get { return instrumentName; }
			set { instrumentName = value; }
		}
	}

	/// <summary>
	/// MetaLyrics: Lyrics
	/// </summary>
	public class MetaLyrics : MetaEvent
	{
		private string lyrics;

		public MetaLyrics() : base(MetaEvent.EVENTTYPE_LYRICS)
		{
			this.Lyrics = "";
		}

		public string Lyrics
		{
			get { return lyrics; }
			set { lyrics = value; }
		}
	}

	/// <summary>
	/// MetaMarker: Marker
	/// </summary>
	public class MetaMarker : MetaEvent
	{
		private string marker;

		public MetaMarker() : base(MetaEvent.EVENTTYPE_MARKER)
		{
			this.Marker = "";
		}

		public string Marker
		{
			get { return marker; }
			set { marker = value; }
		}
	}

	/// <summary>
	/// MetaCuePoint: Cue Point
	/// </summary>
	public class MetaCuePoint : MetaEvent
	{
		private string cuePoint;

		public MetaCuePoint() : base(MetaEvent.EVENTTYPE_CUE_POINT)
		{
			this.CuePoint = "";
		}

		public string CuePoint
		{
			get { return cuePoint; }
			set { cuePoint = value; }
		}
	}

	/// <summary>
	/// MetaMIDIChannelPrefix: MIDI Channel Prefix
	/// </summary>
	public class MetaMIDIChannelPrefix : MetaEvent
	{
		private int channel;

		public MetaMIDIChannelPrefix() : base(MetaEvent.EVENTTYPE_MIDI_CHANNEL_PREFIX)
		{
			this.ChannelPrefix = 0;
		}

		public int ChannelPrefix
		{
			get { return channel; }
			set { channel = value; }
		}
	}

	/// <summary>
	/// MetaMIDIPort: MIDI Port
	/// </summary>
	public class MetaMIDIPort : MetaEvent
	{
		private int port;

		public MetaMIDIPort() : base(MetaEvent.EVENTTYPE_MIDI_PORT)
		{
			this.Port = 0;
		}

		public int Port
		{
			get { return port; }
			set { port = value; }
		}
	}

	/// <summary>
	/// MetaEndOfTrack: End of Track
	/// </summary>
	public class MetaEndOfTrack : MetaEvent
	{
		public MetaEndOfTrack() : base(MetaEvent.EVENTTYPE_END_OF_TRACK)
		{
			// no data: do nothing
		}
	}

	/// <summary>
	/// MetaSetTempo: Set Tempo
	/// </summary>
	public class MetaSetTempo : MetaEvent
	{
		private int tempo;

		public MetaSetTempo() : base(MetaEvent.EVENTTYPE_SET_TEMPO)
		{
			this.Tempo = 0;
		}

		public int Tempo
		{
			get { return tempo; }
			set { tempo = value; }
		}
	}

	/// <summary>
	/// MetaSMPTEOffset: SMPTE Offset
	/// </summary>
	public class MetaSMPTEOffset : MetaEvent
	{
		private int frameType;
		private int hours;
		private int minutes;
		private int seconds;
		private int frames;
		private int fractionalFrames;

		public MetaSMPTEOffset() : base(MetaEvent.EVENTTYPE_SMPTE_OFFSET)
		{
			this.FrameType = 0;
			this.Hours = 0;
			this.Minutes = 0;
			this.Seconds = 0;
			this.Frames = 0;
			this.FractionalFrames = 0;
		}

		public int FrameType
		{
			get { return frameType; }
			set { frameType = value; }
		}

		public int Hours
		{
			get { return hours; }
			set { hours = value; }
		}
		
		public int Minutes
		{
			get { return minutes; }
			set { minutes = value; }
		}

		public int Seconds
		{
			get { return seconds; }
			set { seconds = value; }
		}

		public int Frames
		{
			get { return frames; }
			set { frames = value; }
		}

		public int FractionalFrames
		{
			get { return fractionalFrames; }
			set { fractionalFrames = value; }
		}
	}

	/// <summary>
	/// MetaTimeSignature: Time Signature
	/// </summary>
	public class MetaTimeSignature : MetaEvent
	{
		private int numerator; // beat
		private int denominator; // beat
		private int clocksPerClick; // for metronome
		private int numberOf32ndNotes;

		public MetaTimeSignature() : base(MetaEvent.EVENTTYPE_TIME_SIGNATURE)
		{
			this.Numerator = 4;   // 4/x
			this.Denominator = 4; // x/4
			this.ClocksPerClick = 24; // metronome clicks once every quarter-note
			this.ThirtySecondNotesPerQuarterNote = 8; // 8 32nd notes per 24 MIDI clock signals
		}

		public int Numerator
		{
			get { return numerator; }
			set { numerator = value; }
		}

		public int Denominator
		{
			get { return denominator; }
			set { denominator = value; }
		}

		public int ClocksPerClick
		{
			get { return clocksPerClick; }
			set { clocksPerClick = value; }
		}

		public int ThirtySecondNotesPerQuarterNote
		{
			get { return numberOf32ndNotes; }
			set { numberOf32ndNotes = value; }
		}
	}

	/// <summary>
	/// MetaKeySignature: Key Signature
	/// </summary>
	public class MetaKeySignature : MetaEvent
	{
		private int keys; // number of keys
		private bool isMajor; // scale (mahor or minor)

		public MetaKeySignature() : base(MetaEvent.EVENTTYPE_KEY_SIGNATURE)
		{
			this.Keys = 0;
			this.IsMajorScale = true;
		}

		public int Keys
		{
			get { return keys; }
			set { keys = value; }
		}

		public bool IsMajorScale
		{
			get { return isMajor; }
			set { isMajor = value; }
		}

		public bool IsMinorScale
		{
			get { return (!isMajor); }
			set { isMajor = (!value); }
		}
	}

	/// <summary>
	/// MetaSequencerSpecific: Sequencer Specific
	/// </summary>
	public class MetaSequencerSpecific : MetaEvent
	{
		private byte[] data;

		public MetaSequencerSpecific() : base(MetaEvent.EVENTTYPE_SEQUENCER_SPECIFIC)
		{
			this.Data = null;
		}

		public byte[] Data
		{
			get { return data; }
			set { data = value; }
		}
	}
}
