namespace nkmidi
{
	using System;

	using nkmidi.Meta;

	/// <summary>
	/// Sequence: 1MIDIV[PX\NX
	/// </summary>
	public class Sequence
	{
		private int formatNumber;
		private int numberOfTracks;
		private int timeBase;
		private MIDITrack[] tracks;

		private int startingTempo;

		public Sequence()
		{
			formatNumber = 0;
			numberOfTracks = 0;
			timeBase = 0;
			tracks = null;

			startingTempo = 120;
		}

		public Sequence(SMF smf) : this()
		{
			constructFromSMFFile(smf);
		}

		private void constructFromSMFFile(SMF smf) 
		{
			smf.read();

			formatNumber = (int)smf.FormatNumber;
			numberOfTracks = (int)smf.NumberOfTracks;
			timeBase = (int)smf.TimeBase;
			tracks = smf.Tracks;

			bool flagTempoSet = false;

			// examine meta events
			for (int i=0; i<tracks.Length; i++) 
			{
				MIDIEvent[] events = tracks[i].getEvents();
				for (int j=0; j<events.Length; j++) 
				{
					if ( !events[j].IsMetaEvent ) 
					{
						continue;
					}

					// we only check meta events here
					MetaEvent meta_event = events[j].MetaEvent;

					if ( meta_event.EventType == MetaEvent.EVENTTYPE_SET_TEMPO ) 
					{
						// Set tempo
						if ( !flagTempoSet )
						{
							// starting tempo should be set only once
							MetaSetTempo mev = (MetaSetTempo)meta_event;
							int tempo_usec = mev.Tempo;
							startingTempo = (int)(60 * 1000000 / (double)tempo_usec);

							flagTempoSet = true;
						}
					}
				}
			}
		}



		// accessors and properties

		public int FormatNumber
		{
			get { return formatNumber; }
		}

		public int NumberOfTracks
		{
			get { return numberOfTracks; }
		}

		public int TimeBase
		{
			get { return timeBase; }
		}

		public int StartingTempo
		{
			get { return startingTempo; }
		}

		public MIDITrack[] Tracks
		{
			get { return tracks; }
		}
	}
}
